use glc::vector::{Angle, Vector2f, Vector4f};
use space_crush_common::components::{MeetingPoint, Position};
use specs::{prelude::*, ReadStorage, System, World, WriteExpect};

use crate::resources::Geometry;

#[derive(Default)]
pub struct MeetingPoints {}

#[derive(SystemData)]
pub struct MeetingPointsData<'a> {
    geometry: WriteExpect<'a, Geometry>,
    positions: ReadStorage<'a, Position>,
    meeting_points: ReadStorage<'a, MeetingPoint>,
}

impl<'a> System<'a> for MeetingPoints {
    type SystemData = MeetingPointsData<'a>;

    fn run(&mut self, data: Self::SystemData) {
        let MeetingPointsData {
            mut geometry,
            positions,
            meeting_points,
        } = data;

        gl::enable(gl::BLEND);

        for (position, meeting_point) in (&positions, &meeting_points).join() {
            gl::blend_func(gl::SRC_ALPHA, gl::ONE);
            gl::blend_equation(gl::FUNC_ADD);
            geometry.render_lines(
                Vector4f::new(0.5, 0.5, 0.5, 0.05),
                &create_circle(position.get(), meeting_point.min()),
            );
            geometry.render_lines(
                Vector4f::new(0.5, 0.5, 0.5, 0.05),
                &create_circle(position.get(), meeting_point.max()),
            );
        }

        gl::blend_equation(gl::FUNC_ADD);
        gl::disable(gl::BLEND);
    }
}

fn create_circle(p: &Vector2f, r: f32) -> Vec<Vector2f> {
    let mut points = Vec::new();

    for i in 0..=180 {
        points.push(Vector2f::new(
            p.x + r * Angle::Deg(i as f32 * 2.0).cos(),
            p.y + r * Angle::Deg(i as f32 * 2.0).sin(),
        ));
    }

    points
}
