use specs::{saveload::MarkedBuilder, Builder, Entity, World, WorldExt};

use glc::vector::Vector3f;

use crate::{
    components::{Player, Race},
    misc::{Persistence, WorldPersistence},
};

use super::Error;

#[derive(Clone)]
pub struct PlayerBuilder {
    index: usize,
    race: Race,
    color: Option<Vector3f>,
}

impl PlayerBuilder {
    pub fn build(&mut self, world: &mut World) -> Result<Entity, Error> {
        let index = self.index;
        let race = self.race;
        let color = self.color.take().ok_or(Error::MissingValue("color"))?;

        let player = Player::new(index, race, color);

        let entity = world
            .create_entity()
            .marked::<<WorldPersistence as Persistence>::Marker>()
            .with(player)
            .build();

        self.index += 1;

        Ok(entity)
    }

    pub fn color<V: Into<Vector3f>>(mut self, value: V) -> Self {
        self.color = Some(value.into());

        self
    }
}

impl Default for PlayerBuilder {
    fn default() -> Self {
        Self {
            index: 0,
            race: Race::Fighter,
            color: None,
        }
    }
}
