use serde::{Deserialize, Serialize};
use specs::{Component, HashMapStorage};

use crate::{builder::AsteroidBuilder, misc::FlaggedStorage};

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Asteroid {
    type_: Type,
}

#[derive(Copy, Clone, Debug, Serialize, Deserialize)]
pub enum Type {
    Metal,
    Crystal,
}

/* Asteroid */

impl Asteroid {
    #[inline]
    pub fn builder() -> AsteroidBuilder {
        AsteroidBuilder::default()
    }

    #[inline]
    pub fn type_(&self) -> Type {
        self.type_
    }
}

impl Asteroid {
    #[inline]
    pub(crate) fn new(type_: Type) -> Self {
        Self { type_ }
    }
}

impl Component for Asteroid {
    type Storage = FlaggedStorage<Self, HashMapStorage<Self>>;
}
